**************************************************************************
*  FILE NAME: DJRBROW.PRG                                                *
*  BY: David J. Ramsey                                                   *
*  NOTICE: Copyright 1992 David J. Ramsey                                *
*  DATE: 7/9/92                                                          *
*  DESC: browse of near misses ; prevents duplicate entries              *
*  CALLED BY: Any Program                                                *
*  DATA FILES:  Database required must be opened                         *
*               Indexes required must be opened and set to correct order *
**************************************************************************
* I only ask one thing - don't laugh at my antique, slow, out of date    *
* coding methods.  ADBUS has so many really good programmers that my code*
* is sad compared to yours.  But I have to learn, I'm just slow in figuring*
* out all this new stuff in FOXPRO and CLIPPER.                          *
* I know this is not the slick way to do this stuff, so take the challenge*
* below and show me how to do it the good way. Maybe someone else can    *
* learn from it to.  Surely I am not the only one who is overwhelmed by  *
* 'new feature overload'.                                                *
**************************************************************************                   
** H O W   T O   T R Y  O U T   T H I S   P R O G R A M                 **
*                                                                       **
*  From the command line, use your database, use or set order of index. **
*  Then enter this line of code:                                        **
* do djrbrow with row,col,'pieceofpart','field1','field2','field3','field4'*
*   'pieceofpart' is your NOTFOUND part - pick one that is almost a part *
*  for example if the real entry is '123XYZ' then use '123X' or '123'    *
* use only character or numeric fields for field1-4                      *

*EXAMPLE
*USE f:\appl\fox\data\inventor.dbf index f:\appl\fox\data\inventor.ndx
*(lkey is memvar of not found part)
*DO djrbro WITH 1,3,lkey,3,'ITEM','DESCRIP','SUPPLIER','CLASS'&&,'SHPADD3','STATE'
*                                                                        *
**************************************************************************
*      W I L L   Y O U   T A K E   T H E   C H A L L E N G E ???         *
* Since this is probably a good example of 'weanie?' code, I challenge   *
* everyone who tries this program and screams 'WEANIE CODE ALERT' to write*
* the same program in your best 'NON WEANIE CODE' for either FOXPRO or   *
* CLIPPER and upload it to this bulletin board.  Let everyone see the    *
* best of the 'NO WEANIES WORK HERE' code and at the same time everyone  *
* can see how this can be done using the latest and the greatest.        *
* Seriously, I know this can be done a lot better than I have done it,   *
* so I would truly like to see how to do it better.                      *
*                                                                        *
* If anyone wishes to use my poor example of coding expertise in your own*
* program or modify it so it works better - go for it.                   *
*                                                                        *
**************************************************************************
**               P U R P O S E   O F   P R O G R A M                    **
* This Program combines ADD and EDIT or View into a single procedure     *
*                                                                        *
* With the passed parameters, will automatically create a shaded scroll  *
* box showing the closest matches to an indexed seek.  Will create its   *
* own box, sized according to the fields passed, show as many of these   *
* fields as the screen width will allow, and allows three choices to be  *
* made concerning the fields looked at.  Can be called with only 1 line  *
* of code, and returns the result to your application.                   *
*                                                                        *
* Does not use any of the new FOXPRO specific 'magic commands', so can be*
* converted for use to another xbase version easily.  I need to get to a *
* CLIPPER version now - the parameter passing code is different          *
*                                                                        *
* Using a database indexed on ITEM (part number) you want to check to    *
* see if this item is already in existence.                              *
*                                                                        *
* If the part number was entered correctly, this program is NOT used.    *
*                                                                        *
* If the part number was not found, then you as a programmer, or the user*
* for that matter, have to determine if this really is a new part or was *
* it just entered wrong. Now this program is called.                     *
*                                                                        *
* What this program does is search for a match by looping through a      *
* routine that shortens the item until a match is found.  When a match   *
* is finally found, the index pointer is positioned to the first item    *
* matched using the shortened part number.                               *
*                                                                        *
* A browse opens up displaying the 'matches' to the shortened item.      *
* You can scroll up and down using the <PG UP>,<PG DN>, and up and down  *
* arrow keys.  If the correct spelling of the part number is seen on the *
* screen, then you can use the arrow keys to move down to the correct    *
* part number, PRESS <ENTER> and pass back to the calling program that   *
* the correct item was found.  The index pointer will be positioned to   *
* item.                                                                  *
* If you do not see the correct item on the screen, and want to forget   *
* the whole thing and start over, PRESS <ESC>.                           *
* If by looking at all the near matches, you determine that you really   *
* do have a new entry, then PRESSING <A> key will pass back to the       *
* calling program that you want to add the item you were searching for.  *
*                                                                        *
* What you will find happening on fields that are long, it will be       *
* faster to enter part of the item, and then pick up the item off of the *
* scroll.  I have customers with a part number field that is 26 chars.   *
* It is almost impossible to correctly enter part numbers that are that  *
* long.  They will enter the first 4-5 chars, scroll, and pick off the   *
* screen.                                                                *
*                                                                        *
* If the part is not found at all, the record pointer is placed at the   *
* top of file.                                                           *
**************************************************************************
**              P R O G R A M   R E S T R I C T I O N S                 **
* The part scroll ends at row 18 on the screen.                          *
* If the list of fields you have passed to the screen are longer that the*
* screen has columns for, the fields at the end are dropped off until    *
* they will fit on the screen.                                           *
* Will not display memo fields, works only with Numeric and Character    *
* fields. Obviously, if you have passed a starting position that when    *
* added to the width of the first field is greater than 78, then it won't*
* work. You have to leave enough space to draw the box.                  *
* Code is only written to allow for 80 x 25 screen display.  Changing    *
* to test for screen size or mode would not be difficult.  I just do not *
* have a need for any other screen size.                                 *
*                                                                        *
* Choices at bottom of box may overflow box if total of field lengths    *
* is less than 32 characters. Even if I have a short index field I always*
* add some other fields to give additional information to user, so this  *
* requirement of 32 characters is never a problem.  If it is for you,    *
* then shorten the length of the message at the bottom.                  *                             
**************************************************************************
**  P A R A M E T E R S   P A S S E D   T O   T H I S   P R O G R A M   **
*      scrltop   -  Row position of top corner of box                    *
*      Scrlleft  -  Column position of top left corner of box            *
*      lkey      -  Mem Var of entry (index key) being searched for      *
*      maxopt    -  Number of Options                                    *
*                   One day I will add this     2 = ESC, Select          *
*      code is written for this ->  3 = ESC, Select, Add (new entry)     *
*      f1        -  Field Name of First Field in Scroll                  *
*      f2        -  Field Name of Second Field in Scroll (optional)      *
*      f3        -  Field Name of Third Field in Scroll (optional)       *
*      f4        -  Field Name of Fourth Field in Scroll (optional)      *
**************************************************************************
**      E N D   R E S U L T   O F   T H I S   P R O G R A M             **
* This Program declares a public memvar djchoice.  This memvar will have *
* one of the following:                                                  *
*     S    for RETURN key pressed                                        *
*          for selected (one of the items in the scroll)                 *
*     A    for A key was pressed                                         *
*          for this is a NEW entry that needs to be added                *
*     X    for escape key pressed                                        *   
*          forget it start over                                          *
*                                                                        *
*     From these choices, you can do your own routines for either        *
*     Adding the new entry, using the SELECTED entry, or looping back    *
*     to start over.                                                     *
************************************************************************** 
**   U S I N G    T H I S  P R O G R A M   W I T H   Y O U R   C O D E   *
*  I use this type of format:                                            *
*     djchoice is declared public memvar                                 *                                  *
                                                                         *
*  use your databases and indexes                                        *
*  do while .t.                                                          *
* store Space(len(whatever the key field is)) to mfind                   *
*     @ r,c say 'Enter Part Number (or whatever)                         *
*     @ r,c get mfind                                                    *
*     read                                                               *
*      if mfind = ' '                                                    *
*          return                && no entry, they want to exit program  *
*      endif                                                             *
*      seek mfind                && see if you can find it               *
*      if eof()                  && did not find it1                     *
*      lkey = mfind       && protect the variable mfind in case its new  *                                               *
*      do djrbrow with .......                                           *
*      if djchoice = 'A'                                                 *
*           scatter memvar blank                                         *
*           m.field = lkey                                               *
*           do add routine  (afterall, this is a new entry)              *
*      endif                                                             *
*      if djchoice = 'X'        && did not select one, start over        *
*           loop                                                         *
*      endif                                                             *
*      do your screen display or whatever                                *
*                                                                        *
**************************************************************************

PROCEDURE DJRBROW
PARAMETERS scrltop,scrlleft,lkey,maxopt,f1,f2,f3,f4    
SET TALK OFF 
pcount = PARAMETERS ()
PRIVATE scrlbot, scrlright, scrlheight
PRIVATE currow, lastrec, firstrec, saverec, totf, fl1, fl2,fl3,fl4
PRIVATE l1,l2,l3,l4
Public djchoice
djchoice = ' '

IF pcount >= 5                         && this area is determining how many
   IF TYPE(f1)='N'                     && fields you asked for, how many will
      l1 = LEN(STR(&f1))               && fit, what is the starting position
   ELSE                                && for each field, and the width of the
      l1 = LEN(&f1)                    && box plus the starting position to
   ENDIF                               && be sure that it fits on the screen
   IF scrlleft + l1 < 77 
      totf = 1 
      fl1 = scrlleft
      max_len = fl1 + l1
   ELSE 
      RETURN 
   ENDIF 
ENDIF
   IF pcount >= 6
      IF TYPE(f2)="N"
         l2 = LEN(STR(&f2))
      ELSE 
         l2 = LEN(&f2)
      ENDIF 
      
      IF scrlleft + 1 +l1 + 1 + l2 <77
         totf = 2
         fl2 = fl1 + l1 + 2
         max_len = fl2 + l2
      ENDIF 
   ENDIF 
   IF pcount >= 7 
      IF TYPE(f3)="N"
         l3 = LEN(STR(&f3))
      ELSE 
         l3 = LEN(&f3)
      ENDIF 
      IF scrlleft +l1 + 1 + l2 + 1 + l3 <77
         totf = 3
         fl3 = fl2 +1+ l2 + 2
         max_len = fl3 + l3
      ENDIF 
   ENDIF 
   IF pcount = 8 
      IF TYPE(f4)="N"
         l4 = LEN(STR(&f4))
      ELSE 
         l4 = LEN(&f4)
      ENDIF 
      IF scrlleft+1+l1+l2+1+1+l3 + 1 + l4 < 77
         totf = 4
         fl4 = fl3 + l3 + 2
         max_len = fl4 + l4 + 1
      ENDIF 
   ENDIF 
   
   IF pcount < 6 .or. pcount > 9           && if less than 6, then no fields
      ** can not do                        && passed, if more than 9, then
   ENDIF                                   && more fields passed than I have
                                           && allowed for
   PRIVATE scrlbot, scrlright, scrlheight
   PRIVATE currow, lastrec, firstrec, saverec 
   
   * scroll area coords & dimensions 
   scrlbot = 18
   scrlright = max_len + 2
   scrlheight = scrlbot - (scrltop +4)
   currow = scrltop + 5
   firstrec = RECNO()
   GOTO BOTT 
   lastrec = RECNO()
   GOTO TOP 
   
   hicolor = "N/W"
   locolor = "+W/B"
   
   * display the window   
   SAVE SCREEN TO bro
   CLEAR 

   SET COLOR TO w/n
   @ scrltop+1,scrlleft+1, scrlbot+4, scrlright+2 box "         "
      
   SET COLOR TO +w/b
   @ scrltop,scrlleft-1, scrlbot+3, scrlright box "ͻȺ "
   IF totf >=1
      @ scrltop+2,fl1 SAY SUBSTR(f1,1,l1)
   ENDIF 
   IF totf >=2
      @ scrltop+2,fl2 SAY SUBSTR(f2,1,l2)
   ENDIF 
   IF totf >=3
      @ scrltop+2,fl3 SAY SUBSTR(f3,1,l3)
   ENDIF 
   IF totf=4
      @ scrltop+2,fl4 SAY SUBSTR(f4,1,l4)
   ENDIF 
   
   @ scrltop+1, scrlleft SAY lkey +" Not Found"
   @ scrltop+3, scrlleft SAY REPLICATE('',max_len-scrlleft-1)
   @ scrlbot+1, scrlleft SAY REPLICATE('',max_len-scrlleft-1)
   
   IF max_len - scrlleft > 49
      @ scrlbot+2, scrlleft -2 + ((max_len - 49)/2) SAY "<PgUp><PgDn>  <ESC>=Quit  <A>=Add  <ENTER>=Select"  &&49 chars
   ELSE 
      IF max_len > 49
         @ scrlbot+2, scrlleft + ((max_len-scrlleft - 30)/2) SAY "<ESC>=Quit  <A>=Add  <ENTER>=Select"
      ELSE 
         @ scrlbot+2, scrlleft SAY " <ESC>Quit <A>Add <ENTER>Select"  &&49 chars
      ENDIF 
   ENDIF 
   
   * display first window of records, hilite first one
   SEEK lkey
   DO WHILE LEN(lkey) > 1 .AND. EOF()
      STORE SUBSTR(lkey,1,LEN(lkey)-1) TO lkey
      SEEK lkey
   ENDDO 
   IF .NOT. EOF()
      STORE lkey TO mlkey
   ELSE 
      GO TOP 
   ENDIF 
   
   DO br_disp WITH .t.
   DO br_hidsp
   
   xkey = 0
   
   DO WHILE xkey <> 27 .and. xkey <> 13 .and. xkey <> 65 .and. xkey <> 97
      
      xkey = INKEY(0)
      skey = UPPER(CHR(xkey))
      
      DO CASE 
         
      CASE xkey = 5               && Up arrow key
         IF RECNO() = firstrec
            ?? CHR(7)
         ELSE 
            IF currow > scrltop+5
               DO br_lodsp
               currow = currow - 1
               SKIP -1
               DO br_hidsp
            ELSE 
               SKIP -1
               DO br_disp WITH .t.
               DO br_hidsp
            ENDIF 
         ENDIF 
         
      CASE xkey = 24      && Down arrow key
         IF RECNO() = lastrec
            ?? CHR(7)
         ELSE 
            IF currow < scrlbot
               DO br_lodsp
               currow = currow + 1
               SKIP 
               DO br_hidsp
            ELSE 
               SKIP 2 - scrlheight
               DO br_disp WITH .f.
               DO br_hidsp
            ENDIF 
         ENDIF 
         
      CASE xkey = 3       && PgDn key
         SKIP scrltop + 4-currow + (2*scrlheight) -1
         IF EOF()
            ?? CHR(7)
            SKIP -scrlheight
            DO br_disp WITH .f.
            DO br_hidsp
         ELSE 
            SKIP 1 -scrlheight
            DO br_disp WITH .t.
            DO br_hidsp
         ENDIF 
         
      CASE xkey = 18      && PgUp key
         SKIP scrltop -4- currow - scrlheight
         IF BOF()
            ?? CHR(7)
            GOTO TOP 
            DO br_disp WITH .t.
            DO br_hidsp
         ELSE 
            DO br_disp WITH .t.
            DO br_hidsp
         ENDIF 
         
      CASE xkey = 1       && Home key
         GOTO TOP 
         DO br_disp WITH .t.
         DO br_hidsp
         
      CASE xkey = 6       && End key
         GOTO BOTT 
         SKIP 1 - scrlheight
         IF BOF()
            GOTO TOP 
         ENDIF 
         DO br_disp WITH .f.
         DO br_hidsp
         
      ENDCASE 
   ENDDO 
    
   DO CASE 
   CASE xkey = 27                                 && escape NO CHOICE SELECTED
      store 'X' to choice
      @ 22,2 say 'You picked X - forget it'       && this is for demo purposes
      wait                                        && this is for demo purposes
   CASE xkey = 13                                 && return   PICKED THIS ONE
      store 'S' to choice
      @ 22,2 say 'You picked S for select this'   &&this is for demo purposes
      wait                                        &&this is for demo purposes
   CASE xkey = 65 .or. xkey = 97                  && WANTS TO ADD A NEW ENTRY
      store 'A' to choice
      @ 22,2 say 'You picked A - Add this entry'  &&this is for demo purposes
      wait                                        &&this is for demo purposes
   ENDCASE 
   RESTORE SCREEN FROM bro
   SET COLOR TO &locolor
   
   RETURN         &&whatever the djchoice was   select, add, escape
   
   *** br_hidsp: hilites prompt line at currow
   
   PROC br_hidsp
   SET COLOR TO &hicolor
   DO CASE 
   CASE totf = 1
      @ currow, fl1 SAY &f1
   CASE totf = 2
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
   CASE totf = 3
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
      @ currow, fl3 SAY &f3
   CASE totf = 4
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
      @ currow, fl3 SAY &f3
      @ currow, fl4 SAY &f4
   ENDCASE 
   RETURN 
   
   *** br_lodsp: lolites prompt line at currow
   
   PROC br_lodsp
   SET COLOR TO &locolor
   DO CASE 
   CASE totf = 1
      @ currow, fl1 SAY &f1
   CASE totf = 2
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
   CASE totf = 3
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
      @ currow, fl3 SAY &f3
   CASE totf = 4
      @ currow, fl1 SAY &f1
      @ currow, fl2 SAY &f2
      @ currow, fl3 SAY &f3
      @ currow, fl4 SAY &f4
   ENDCASE 
   RETURN 
   
   *** br_disp : displays a window full of records.

   PROC br_disp
   PARAM set_to_top
   
   PRIVATE thisrow, toprec
   
   SET COLOR TO &locolor
   
   toprec = RECNO()
   thisrow = scrltop+5
   DO WHILE .not. EOF() .and. thisrow <= scrlbot
      DO CASE 
      CASE totf = 1
         @ thisrow, fl1 SAY &f1
      CASE totf = 2
         @ thisrow, fl1 SAY &f1
         @ thisrow, fl2 SAY &f2
      CASE totf = 3
         @ thisrow, fl1 SAY &f1
         @ thisrow, fl2 SAY &f2
         @ thisrow, fl3 SAY &f3
      CASE totf = 4
         @ thisrow, fl1 SAY &f1
         @ thisrow, fl2 SAY &f2
         @ thisrow, fl3 SAY &f3
         @ thisrow, fl4 SAY &f4
      ENDCASE 
      SKIP 
      thisrow = thisrow + 1
   ENDDO 
   
   IF set_to_top
      GOTO toprec
      currow = scrltop+4 
   ELSE 
      SKIP -1
      currow = thisrow -1
   ENDIF 
   
   RETURN 
*Formatted by: ToolBox Ver. 1.2  on 7/9/92 at 10:56 AM.
